@@ -328,6 +328,16 @@ int AVSC_CC avs_is_writable(const AVS_VideoFrame * p)
   return 0;
 }
 
+// V9
+extern "C"
+int AVSC_CC avs_is_property_writable(const AVS_VideoFrame * p)
+{
+  if (p->refcount == 1) {
+    return 1;
+  }
+  return 0;
+}
+
 extern "C"
 BYTE * AVSC_CC avs_get_write_ptr_p(const AVS_VideoFrame * p, int plane)
 {
@@ -385,7 +395,7 @@ void AVSC_CC avs_copy_frame_props(AVS_ScriptEnvironment * p, const AVS_VideoFram
 {
   p->error = 0;
   try {
-    p->env->copyFrameProps(*(const PVideoFrame*)(src), *(PVideoFrame*)(dst));
+    p->env->copyFrameProps(*(const PVideoFrame*)&src, *(PVideoFrame*)&dst);
   }
   catch (const AvisynthError& err) {
     p->error = err.msg;
@@ -397,7 +407,7 @@ const AVS_Map * AVSC_CC avs_get_frame_props_ro(AVS_ScriptEnvironment * p, const
 {
   p->error = 0;
   try {
-    return (const AVS_Map*)(p->env->getFramePropsRO(*(const PVideoFrame*)frame));
+    return (const AVS_Map*)(p->env->getFramePropsRO(*(const PVideoFrame*)&frame));
   }
   catch (const AvisynthError& err) {
     p->error = err.msg;
@@ -410,7 +420,7 @@ AVS_Map * AVSC_CC avs_get_frame_props_rw(AVS_ScriptEnvironment * p, AVS_VideoFra
 {
   p->error = 0;
   try {
-    return (AVS_Map*)(p->env->getFramePropsRW(*(PVideoFrame*)frame));
+    return (AVS_Map*)(p->env->getFramePropsRW(*(PVideoFrame*)&frame));
   }
   catch (const AvisynthError& err) {
     p->error = err.msg;
@@ -632,7 +642,7 @@ int AVSC_CC avs_prop_set_frame(AVS_ScriptEnvironment * p, AVS_Map * map, const c
   // length = -1 -> auto strlen
   p->error = 0;
   try {
-    return (p->env->propSetFrame((AVSMap*)map, key, *(PVideoFrame*)frame, append));
+    return (p->env->propSetFrame((AVSMap*)map, key, *(PVideoFrame*)&frame, append));
   }
   catch (const AvisynthError& err) {
     p->error = err.msg;
@@ -893,22 +903,17 @@ extern "C"
 void AVSC_CC avs_copy_value(AVS_Value * dest, AVS_Value src)
 {
   // true: don't copy array elements recursively
-#ifdef NEW_AVSVALUE
   new(dest) AVSValue(*(const AVSValue*)&src, true);
-#else
-  new(dest) AVSValue(*(const AVSValue*)&src);
-#endif
 }
 
 extern "C"
 void AVSC_CC avs_release_value(AVS_Value v)
 {
-#ifdef NEW_AVSVALUE
   if (((AVSValue*)&v)->IsArray()) {
     // signing for destructor: don't free array elements
     ((AVSValue*)&v)->MarkArrayAsC();
   }
-#endif
+
   ((AVSValue*)&v)->~AVSValue();
 }
 
@@ -1173,6 +1178,20 @@ int AVSC_CC avs_make_writable(AVS_ScriptEnvironment * p, AVS_VideoFrame * *pvf)
   return -1;
 }
 
+// Since V9
+extern "C"
+int AVSC_CC avs_make_property_writable(AVS_ScriptEnvironment * p, AVS_VideoFrame * *pvf)
+{
+  p->error = 0;
+  try {
+    return p->env->MakePropertyWritable((PVideoFrame*)(pvf));
+  }
+  catch (const AvisynthError& err) {
+    p->error = err.msg;
+  }
+  return -1;
+}
+
 extern "C"
 void AVSC_CC avs_bit_blt(AVS_ScriptEnvironment * p, BYTE * dstp, int dst_pitch, const BYTE * srcp, int src_pitch, int row_size, int height)
 {
