@@ -61,6 +61,8 @@
 #include <map>
 #include <string>
 #include <utility>
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 #ifndef MINGW_HAS_SECURE_API
 #define sprintf_s sprintf
@@ -297,7 +299,6 @@ extern const AVSFunction Script_functions[] = {
   { "VarExist", BUILTIN_FUNC_PREFIX, "s", VarExist }, // 180606-
 
 
-#ifdef NEW_AVSVALUE
     // Creates script array from zero or more anything.
     // Direct array constant syntax e.g. x = [arg1,arg2,...] is translated to x = Array(arg1,arg2,...)
   { "Array", BUILTIN_FUNC_PREFIX, ".*", ArrayCreate },
@@ -307,13 +308,14 @@ extern const AVSFunction Script_functions[] = {
     // classic array indexing background helper: e.g. a[3,4] -> ArrayGet(a, [2,3])
   { "ArrayGet",  BUILTIN_FUNC_PREFIX, ".i+", ArrayGet }, // .+i+ syntax is not possible.
     // length can be zero
-  { "ArraySize", BUILTIN_FUNC_PREFIX, ".*", ArraySize },
+  { "ArraySize", BUILTIN_FUNC_PREFIX, ".", ArraySize },
+  { "ArrayIns",  BUILTIN_FUNC_PREFIX, "..i+", ArrayIns, (void*)0 },
+  { "ArrayAdd",  BUILTIN_FUNC_PREFIX, "..i*", ArrayIns, (void*)1 },
+  { "ArraySet",  BUILTIN_FUNC_PREFIX, "..i+", ArrayIns, (void*)2 },
+  { "ArrayDel",  BUILTIN_FUNC_PREFIX, ".i+", ArrayIns, (void*)3 },
   /*
-  { "ArrayAdd",  BUILTIN_FUNC_PREFIX, ".i*", ArrayAdd },
-  { "ArrayDel",  BUILTIN_FUNC_PREFIX, ".i", ArrayDel },
-  { "ArrayIns",  BUILTIN_FUNC_PREFIX, ".i", ArrayDel },
+  { "IsArrayOf", BUILTIN_FUNC_PREFIX, ".s", IsArrayOf },
   */
-#endif
   { 0 }
 };
 
@@ -1369,7 +1371,7 @@ AVSValue AudioLengthS(AVSValue args, void*, IScriptEnvironment* env) {
 #ifdef AVS_WINDOWS
     return env->SaveString(_i64toa(VI(args[0]).num_audio_samples, s, 10));
 #else
-    sprintf(s,"%ld",VI(args[0]).num_audio_samples);
+    sprintf(s, "%" PRId64, VI(args[0]).num_audio_samples);
     return env->SaveString(s);
 #endif
 }
@@ -2160,18 +2162,8 @@ AVSValue VarExist(AVSValue args, void*, IScriptEnvironment* env)
 }
 
 
-#ifdef NEW_AVSVALUE
-
 AVSValue ArrayCreate(AVSValue args, void*, IScriptEnvironment* env)
 {
-  if (args[0].IsArray() && args[0].ArraySize() == 1) {
-    AVSValue arg0 = args[0];
-    if (arg0.ArraySize() == 0)
-      return new AVSValue(nullptr, 0); // special case: zero length array
-    else
-      return args[0];
-  }
-else
     return args[0];
 }
 
@@ -2180,55 +2172,144 @@ AVSValue IsArray(AVSValue args, void*, IScriptEnvironment* env) { return args[0]
 AVSValue ArrayGet(AVSValue args, void*, IScriptEnvironment* env)
 {
   // signature .i+
-  // parameters: [0] array to index; [1] one or more integer indexes
+  // parameters: [0] array to index; [1] one or more integer indexes or a string
   if (!args[0].IsArray())
-    env->ThrowError("Array required.");
+    env->ThrowError("ArrayGet: array type required.");
   const int size = args[0].ArraySize();
   if (args[1].IsString()) {
     // associative search
     // { {"a", element1}, { "b", element2 }, etc..}
-    const char *tag = args[1].AsString();
-    for (int i = 0; i < size; i++)
-    {
-      AVSValue currentTagValue = args[0][i]; // two elements e.g. { "b", element2 }
-      if(!currentTagValue.IsArray())
-        env->ThrowError("Array must contain array[string, any] elements for dictionary lookup");
-      if(currentTagValue.ArraySize() < 2)
-        env->ThrowError("Internal array must have at least two elements (tag, value)");
-      AVSValue currentTag = currentTagValue[0];
-      if (currentTag.IsString() && !lstrcmpi(currentTag.AsString(), tag))
-      {
-        return currentTagValue[1];
-      }
-    }
-    return AVSValue(); // undefined
+const char* tag = args[1].AsString();
+for (int i = 0; i < size; i++)
+{
+  AVSValue currentTagValue = args[0][i]; // two elements e.g. { "b", element2 }
+  if (!currentTagValue.IsArray())
+    env->ThrowError("ArrayGet: Array must contain array[string, any] elements for dictionary lookup");
+  if (currentTagValue.ArraySize() < 2)
+    env->ThrowError("ArrayGet: Internal array must have at least two elements (tag, value)");
+  AVSValue currentTag = currentTagValue[0];
+  if (currentTag.IsString() && !lstrcmpi(currentTag.AsString(), tag))
+  {
+    return currentTagValue[1];
+  }
+}
+return AVSValue(); // undefined
   }
   else if (args[1].IsArray()) {
-    AVSValue indexes = args[1];
-    AVSValue currentValue = args[0];
-    int index_count = indexes.ArraySize(); // array of parameters. a[1,2] -> [1,2]
-    if(index_count == 0)
-      env->ThrowError("ArrayGet: no index specified");
-    for (int i = 0; i < index_count; i++)
-    {
-      if(!currentValue.IsArray())
-        env->ThrowError("ArrayGet: not an array. Index=%d", i);
-      int currentIndex = indexes[i].AsInt();
-      if(currentIndex < 0 || currentIndex >= currentValue.ArraySize())
-        env->ThrowError("Array index out of range. Problematic index count: %d", i+1);
-      currentValue = currentValue[currentIndex];
-    }
-    return currentValue;
+  AVSValue indexes = args[1];
+  AVSValue currentValue = args[0];
+  int index_count = indexes.ArraySize(); // array of parameters. a[1,2] -> [1,2]
+  if (index_count == 0)
+    env->ThrowError("ArrayGet: no index specified");
+  for (int i = 0; i < index_count; i++)
+  {
+    if (!currentValue.IsArray())
+      env->ThrowError("ArrayGet: not an array. Index=%d", i);
+    int currentIndex = indexes[i].AsInt();
+    if (currentIndex < 0 || currentIndex >= currentValue.ArraySize())
+      env->ThrowError("ArrayGet: Array index out of range. Problematic index count: %d", i + 1);
+    currentValue = currentValue[currentIndex];
   }
-  env->ThrowError("Invalid array index, must be integer or string, or comma separated integers");
+  return currentValue;
+  }
+  env->ThrowError("ArrayGet: Invalid array index, must be integer or string, or comma separated integers");
   return AVSValue(); // undefined
 }
 
 AVSValue ArraySize(AVSValue args, void*, IScriptEnvironment* env)
 {
-  // func signature: ".*"
-  if (!args[0][0].IsArray())
-    env->ThrowError("Parameter must be array");
-  return args[0][0].ArraySize();
+  // func signature: "."
+  if (!args[0].IsArray())
+    env->ThrowError("ArraySize: parameter must be an array");
+  return args[0].ArraySize();
 }
-#endif
+
+AVSValue ArrayIns(AVSValue args, void* user_data, IScriptEnvironment* env)
+{
+  int mode = (int)(intptr_t)user_data;
+  enum ArrayMode {
+    INSERT = 0,
+    APPEND = 1,
+    REPLACE = 2,
+    DEL = 3
+  };
+  // signature .. and ..i
+  // parameters:
+  // [0] array to modify;
+  // [1] element to insert (ArrayAdd, ArrayIns and ArraySet) [2] inserting index(es) (ArrayIns, ArraySet)
+  // or [1] delete index(es) ArrayDel
+
+  const char* funcname = mode == DEL ? "ArrayDel" : mode == REPLACE ? "ArraySet" : mode == APPEND ? "ArrayAdd" : "ArrayIns";
+
+  if (!args[0].IsArray())
+    env->ThrowError("%s error: array type required.", funcname);
+
+  const auto orig_size = args[0].ArraySize();
+
+  const int index_param_pos = mode == DEL ? 1 : 2;
+  AVSValue indexes = args[index_param_pos];
+  int index_count = indexes.ArraySize(); // array of parameters. a[1,2] -> [1,2]
+
+  if (mode == INSERT || mode == REPLACE || mode == DEL) {
+    if (index_count == 0)
+      env->ThrowError("%s: no index specified", funcname);
+  }
+
+  const int new_size =
+    mode == DEL && index_count == 1 ? orig_size - 1 :
+    mode == APPEND && index_count == 0 ? orig_size + 1 :
+    mode == INSERT && index_count == 1 ? orig_size + 1 :
+    orig_size; // replace and recurside other cases
+
+  std::vector<AVSValue> new_val(new_size);
+
+  int action_pos;
+  if (mode == APPEND)
+    action_pos = orig_size; // at the end
+  else
+    action_pos = indexes[0].AsInt();
+
+  // copy before insertion/replace point
+  for (int i = 0; i < action_pos; i++)
+    new_val[i] = args[0][i]; // avs+: automatic deep copy
+
+  if (
+    ((mode == REPLACE || mode == INSERT || mode == DEL) && index_count > 1) ||
+    ((mode == APPEND) && index_count >= 1))
+  {
+    int current_index = indexes[0].AsInt();
+    // for multi-level array recursion is needed because there is no exact reference to an inner element
+    if (mode == DEL) {
+      AVSValue params[2] = { args[0][current_index], index_count <= 1 ? AVSValue(nullptr, 0) : AVSValue(&indexes[1], index_count - 1) };
+      new_val[current_index] = env->Invoke(funcname, AVSValue(params, 2)); // recursively
+    }
+    else {
+      AVSValue params[3] = { args[0][current_index], args[1], index_count <= 1 ? AVSValue(nullptr, 0) : AVSValue(&indexes[1], index_count - 1) };
+      new_val[current_index] = env->Invoke(funcname, AVSValue(params, 3)); // recursively
+    }
+    mode = REPLACE;
+  }
+  else if (mode != DEL) {
+    new_val[action_pos] = args[1];
+  }
+
+  // copy from after insertion/replace/delete point
+  if (mode == DEL) {
+    for (int i = action_pos + 1; i < orig_size; i++)
+      new_val[i - 1] = args[0][i]; // avs+: automatic deep copy
+  }
+  else if (mode == REPLACE) {
+    for (int i = action_pos+1; i < orig_size; i++)
+      new_val[i] = args[0][i]; // avs+: automatic deep copy
+  }
+  else {
+    for (int i = action_pos; i < orig_size; i++)
+      new_val[i + 1] = args[0][i]; // avs+: automatic deep copy
+  }
+
+  if(new_size == 0)
+    return AVSValue(nullptr, 0); // zero array
+
+  return AVSValue(new_val.data(), new_size);
+}
+
