@@ -78,20 +78,22 @@ extern const AVSFunction Conditional_filters[] = {
   { "propSetFloat", BUILTIN_FUNC_PREFIX, "csn[mode]i", SetProperty::Create, (void*)2 }, // forced check for float
   { "propSetString", BUILTIN_FUNC_PREFIX, "csn[mode]i", SetProperty::Create, (void*)3 }, // forced check for string
   { "propSetArray", BUILTIN_FUNC_PREFIX, "csn", SetProperty::Create, (void*)4 }, // no mode parameter, full entry refresh
+  { "propSetClip", BUILTIN_FUNC_PREFIX, "csn[mode]i", SetProperty::Create, (void*)5 }, // forced check for clip
 
   // value from direct data
   { "propSet", BUILTIN_FUNC_PREFIX, "csi[mode]i", SetProperty::Create, (void*)10 },
   { "propSet", BUILTIN_FUNC_PREFIX, "csf[mode]i", SetProperty::Create, (void*)11 },
   { "propSet", BUILTIN_FUNC_PREFIX, "css[mode]i", SetProperty::Create, (void*)12 },
-#ifdef NEW_AVSVALUE
   { "propSet", BUILTIN_FUNC_PREFIX, "csa", SetProperty::Create, (void*)13 }, // no mode parameter, full entry refresh
-#endif
+  { "propSet", BUILTIN_FUNC_PREFIX, "csc[mode]i", SetProperty::Create, (void*)14 },
 
-  { "propDelete", BUILTIN_FUNC_PREFIX, "cs", DeleteProperty::Create },
+  { "propDelete", BUILTIN_FUNC_PREFIX, "cs+", DeleteProperty::Create },
   { "propClearAll", BUILTIN_FUNC_PREFIX, "c", ClearProperties::Create },
 
   { "propShow", BUILTIN_FUNC_PREFIX, "c[size]i[showtype]b", ShowProperties::Create },
 
+  { "propCopy", BUILTIN_FUNC_PREFIX, "cc[merge]b[props]s+[exclude]b", CopyProperties::Create},
+
   { 0 }
 };
 
@@ -111,6 +113,7 @@ ConditionalSelect::ConditionalSelect(PClip _child, AVSValue _script,
   GenericVideoFilter(_child), script(_script),
   num_args(_num_args), child_array(_child_array), show(_show), local(_local) {
 
+  child_devs = DEV_TYPE_ANY;
   for (int i=0; i<num_args; i++) {
     const VideoInfo& vin = child_array[i]->GetVideoInfo();
 
@@ -125,6 +128,12 @@ ConditionalSelect::ConditionalSelect(PClip _child, AVSValue _script,
 
     if (vi.num_frames < vin.num_frames) // Max of all clips
       vi.num_frames = vin.num_frames;
+
+    child_devs &= GetDeviceTypes(child_array[i]);
+  }
+
+  if (child_devs == 0) {
+    env->ThrowError("ConditionalSelect: No common device among sources!");
   }
 }
 
@@ -136,7 +145,16 @@ ConditionalSelect::~ConditionalSelect() {
 int __stdcall ConditionalSelect::SetCacheHints(int cachehints, int frame_range)
 {
   AVS_UNUSED(frame_range);
-  return cachehints == CACHE_GET_MTMODE ? MT_NICE_FILTER : 0;
+  switch (cachehints)
+  {
+  case CACHE_GET_MTMODE:
+    return MT_NICE_FILTER;
+  case CACHE_GET_DEV_TYPE:
+    return child_devs;
+  case CACHE_GET_CHILD_DEV_TYPE:
+    return DEV_TYPE_ANY;
+  }
+  return 0;  // We do not pass cache requests upwards.
 }
 
 PVideoFrame __stdcall ConditionalSelect::GetFrame(int n, IScriptEnvironment* env)
@@ -313,6 +331,12 @@ ConditionalFilter::ConditionalFilter(PClip _child, PClip _source1, PClip _source
     vi.fps_numerator = vi1.fps_numerator;
     vi.nchannels = vi1.nchannels;
     vi.sample_type = vi1.sample_type;
+
+    child_devs = (GetDeviceTypes(source1) & GetDeviceTypes(source2));
+
+    if (child_devs == 0) {
+      env->ThrowError("ConditionalFilter: The two sources must support the same device!");
+    }
   }
 
 const char* const t_TRUE="TRUE";
@@ -321,7 +345,16 @@ const char* const t_FALSE="FALSE";
 int __stdcall ConditionalFilter::SetCacheHints(int cachehints, int frame_range)
 {
   AVS_UNUSED(frame_range);
-  return cachehints == CACHE_GET_MTMODE ? MT_NICE_FILTER : 0;
+  switch (cachehints)
+  {
+  case CACHE_GET_MTMODE:
+    return MT_NICE_FILTER;
+  case CACHE_GET_DEV_TYPE:
+    return child_devs;
+  case CACHE_GET_CHILD_DEV_TYPE:
+    return DEV_TYPE_ANY;
+  }
+  return 0;  // We do not pass cache requests upwards.
 }
 
 PVideoFrame __stdcall ConditionalFilter::GetFrame(int n, IScriptEnvironment* env)
@@ -542,7 +575,14 @@ ScriptClip::ScriptClip(PClip _child, AVSValue  _script, bool _show, bool _only_e
 
 int __stdcall ScriptClip::SetCacheHints(int cachehints, int frame_range)
 {
-  return cachehints == CACHE_GET_MTMODE ? MT_NICE_FILTER : 0;
+  switch (cachehints)
+  {
+  case CACHE_GET_MTMODE:
+    return MT_NICE_FILTER;
+  case CACHE_GET_DEV_TYPE:
+    return (child->GetVersion() >= 5) ? child->SetCacheHints(CACHE_GET_DEV_TYPE, 0) : 0;
+  }
+  return 0;  // We do not pass cache requests upwards.
 }
 
 PVideoFrame __stdcall ScriptClip::GetFrame(int n, IScriptEnvironment* env)
