@@ -35,11 +35,7 @@
 #include "edit.h"
 #include "../convert/convert_audio.h"
 #include "../core/internal.h"
-#ifdef INTEL_INTRINSICS
-#include "intel/merge_sse.h"
-#else
 #include "merge.h"
-#endif
 #include <climits>
 #include <cmath>
 
@@ -59,15 +55,17 @@
 ***** Declare index of new filters for Avisynth's filter engine *****
 ********************************************************************/
 
+enum { FADE_MODE_OUT0, FADE_MODE_OUT, FADE_MODE_OUT2, FADE_MODE_IN0, FADE_MODE_IN, FADE_MODE_IN2, FADE_MODE_IO0, FADE_MODE_IO, FADE_MODE_IO2 };
+
 extern const AVSFunction Edit_filters[] = {
-  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cff",          Trim::CreateA, (void*)Trim::Default}, // start time, end time
-  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cf",           Trim::CreateA, (void*)Trim::Invalid}, // Throw Invalid argument to AudioTrim
-  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cf[length]f",  Trim::CreateA, (void*)Trim::Length},  // start time, duration
-  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cf[end]f",     Trim::CreateA, (void*)Trim::End},     // start time, end time
-  { "Trim", BUILTIN_FUNC_PREFIX, "cii[pad]b",         Trim::Create,  (void*)Trim::Default}, // first frame, last frame[, pad audio]
-  { "Trim", BUILTIN_FUNC_PREFIX, "ci[pad]b",          Trim::Create,  (void*)Trim::Invalid}, // Throw Invalid argument to Trim
-  { "Trim", BUILTIN_FUNC_PREFIX, "ci[length]i[pad]b", Trim::Create,  (void*)Trim::Length},  // first frame, frame count[, pad audio]
-  { "Trim", BUILTIN_FUNC_PREFIX, "ci[end]i[pad]b",    Trim::Create,  (void*)Trim::End},     // first frame, last frame[, pad audio]
+  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cff[cache]b",          Trim::CreateA, (void*)Trim::Default}, // start time, end time
+  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cf[cache]b",           Trim::CreateA, (void*)Trim::Invalid}, // Throw Invalid argument to AudioTrim because 4 parameters expected
+  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cf[length]f[cache]b",  Trim::CreateA, (void*)Trim::Length},  // start time, duration
+  { "AudioTrim", BUILTIN_FUNC_PREFIX, "cf[end]f[cache]b",     Trim::CreateA, (void*)Trim::End},     // start time, end time
+  { "Trim", BUILTIN_FUNC_PREFIX, "cii[pad]b[cache]b",         Trim::Create,  (void*)Trim::Default}, // first frame, last frame[, pad audio]
+  { "Trim", BUILTIN_FUNC_PREFIX, "ci[pad]b[cache]b",          Trim::Create,  (void*)Trim::Invalid}, // Throw Invalid argument to Trim because 5 parameters expected
+  { "Trim", BUILTIN_FUNC_PREFIX, "ci[length]i[pad]b[cache]b", Trim::Create,  (void*)Trim::Length},  // first frame, frame count[, pad audio]
+  { "Trim", BUILTIN_FUNC_PREFIX, "ci[end]i[pad]b[cache]b",    Trim::Create,  (void*)Trim::End},     // first frame, last frame[, pad audio]
   { "FreezeFrame", BUILTIN_FUNC_PREFIX, "ciii", FreezeFrame::Create },           // first frame, last frame, source frame
   { "DeleteFrame", BUILTIN_FUNC_PREFIX, "ci+", DeleteFrame::Create },            // frame #
   { "DuplicateFrame",  BUILTIN_FUNC_PREFIX, "ci+", DuplicateFrame::Create },      // frame #
@@ -79,15 +77,15 @@ extern const AVSFunction Edit_filters[] = {
   { "AudioDub",   BUILTIN_FUNC_PREFIX, "cc", AudioDub::Create, (void*)0},          // video src, audio src
   { "AudioDubEx", BUILTIN_FUNC_PREFIX, "cc", AudioDub::Create, (void*)1},        // video! src, audio! src
   { "Reverse",  BUILTIN_FUNC_PREFIX, "c", Reverse::Create },                      // plays backwards
-  { "FadeOut0", BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeOut0},       // # frames[, color][, fps]
-  { "FadeOut",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeOut},         // # frames[, color][, fps]
-  { "FadeOut2", BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeOut2},       // # frames[, color][, fps]
-  { "FadeIn0",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeIn0},         // # frames[, color][, fps]
-  { "FadeIn",   BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeIn},           // # frames[, color][, fps]
-  { "FadeIn2",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeIn2},         // # frames[, color][, fps]
-  { "FadeIO0",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeIO0},         // # frames[, color][, fps]
-  { "FadeIO",   BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeIO},           // # frames[, color][, fps]
-  { "FadeIO2",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f", Create_FadeIO2},         // # frames[, color][, fps]
+  { "FadeOut0", BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_OUT0 },
+  { "FadeOut",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_OUT },
+  { "FadeOut2", BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_OUT2 },
+  { "FadeIn0",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_IN0 },
+  { "FadeIn",   BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_IN2 },
+  { "FadeIn2",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_IN2 },
+  { "FadeIO0",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_IO0 },
+  { "FadeIO",   BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_IO },
+  { "FadeIO2",  BUILTIN_FUNC_PREFIX, "ci[color]i[fps]f[color_yuv]i[colors]f+", Create_Fade, (void*)FADE_MODE_IO2 },
   { "Loop",     BUILTIN_FUNC_PREFIX, "c[times]i[start]i[end]i", Loop::Create },      // number of loops, first frame, last frames
   { NULL }
 };
@@ -112,6 +110,10 @@ int __stdcall NonCachedGenericVideoFilter::SetCacheHints(int cachehints, int fra
       return 1;
     case CACHE_GET_MTMODE:
       return MT_NICE_FILTER;
+
+    case CACHE_GET_DEV_TYPE:
+      return (child->GetVersion() >= 5) ? child->SetCacheHints(CACHE_GET_DEV_TYPE, 0) : 0;
+
     default:
       return GenericVideoFilter::SetCacheHints(cachehints, frame_range);
   }
@@ -122,8 +124,8 @@ int __stdcall NonCachedGenericVideoFilter::SetCacheHints(int cachehints, int fra
  *******   AudioTrim Filter   ******
  ******************************/
 
-Trim::Trim(double starttime, double endtime, PClip _child, trim_mode_e mode, IScriptEnvironment* env)
- : GenericVideoFilter(_child)
+Trim::Trim(double starttime, double endtime, PClip _child, trim_mode_e mode, bool _cache, IScriptEnvironment* env)
+ : GenericVideoFilter(_child), cache(_cache)
 {
   int64_t esampleno = 0;
 
@@ -187,7 +189,9 @@ AVSValue __cdecl Trim::CreateA(AVSValue args, void* user_arg, IScriptEnvironment
     if (mode == Trim::Invalid)
         env->ThrowError("Script error: Invalid arguments to function \"AudioTrim\"");
 
-    return new Trim(args[1].AsFloat(), args[2].AsFloat(), args[0].AsClip(), mode, env);
+    const bool cache = args[3].AsBool(true);
+
+    return new Trim(args[1].AsFloat(), args[2].AsFloat(), args[0].AsClip(), mode, cache, env);
 }
 
 
@@ -195,8 +199,8 @@ AVSValue __cdecl Trim::CreateA(AVSValue args, void* user_arg, IScriptEnvironment
  *******   Trim Filter   ******
  ******************************/
 
-Trim::Trim(int _firstframe, int _lastframe, bool _padaudio, PClip _child, trim_mode_e mode, IScriptEnvironment* env)
- : GenericVideoFilter(_child)
+Trim::Trim(int _firstframe, int _lastframe, bool _padaudio, PClip _child, trim_mode_e mode, bool _cache, IScriptEnvironment* env)
+ : GenericVideoFilter(_child), cache(_cache)
 {
   int lastframe = 0;
 
@@ -292,7 +296,9 @@ AVSValue __cdecl Trim::Create(AVSValue args, void* user_arg, IScriptEnvironment*
     if (mode == Trim::Invalid)
         env->ThrowError("Script error: Invalid arguments to function \"Trim\"");
 
-    return new Trim(args[1].AsInt(), args[2].AsInt(), args[3].AsBool(true), args[0].AsClip(), mode, env);
+    const bool cache = args[4].AsBool(true);
+
+    return new Trim(args[1].AsInt(), args[2].AsInt(), args[3].AsBool(true), args[0].AsClip(), mode, cache, env);
 }
 
 
@@ -493,6 +499,8 @@ Splice::Splice(PClip _child1, PClip _child2, bool realign_sound, bool _passCache
     env->ThrowError("Splice: Maximum number of frames exceeded.");
 
   vi.num_audio_samples = audio_switchover_point + vi2.num_audio_samples;
+
+  child_devs = (GetDeviceTypes(child) & GetDeviceTypes(child2));
 }
 
 
@@ -536,6 +544,8 @@ int Splice::SetCacheHints(int cachehints,int frame_range)
     return 1;
   case CACHE_GET_MTMODE:
     return MT_NICE_FILTER;
+  case CACHE_GET_DEV_TYPE:
+    return child_devs;
   default:
     if (passCache) {
       child2->SetCacheHints(cachehints, frame_range);
@@ -876,6 +886,8 @@ int __stdcall AudioDub::SetCacheHints(int cachehints,int frame_range)
     return 1;
   case CACHE_GET_MTMODE:
     return MT_NICE_FILTER;
+  case CACHE_GET_DEV_TYPE:
+    return (vchild->GetVersion() >= 5) ? vchild->SetCacheHints(CACHE_GET_DEV_TYPE, 0) : 0;
   default:
     return 0;
   }
@@ -1064,104 +1076,52 @@ AVSValue __cdecl Loop::Create(AVSValue args, void*, IScriptEnvironment* env)
  *****************************/
 
 
-PClip __cdecl ColorClip(PClip a, int duration, int color, float fps, IScriptEnvironment* env) {
+PClip __cdecl ColorClip(PClip a, int duration, AVSValue color, AVSValue color_yuv, AVSValue colors, float fps, IScriptEnvironment* env) {
   if (a->GetVideoInfo().HasVideo())
   {
-    AVSValue blackness_args[] = { a, duration, color };
-    static const char* const arg_names[3] = { 0, 0, "color" };
-    return env->Invoke("Blackness", AVSValue(blackness_args, 3), arg_names ).AsClip();
+    AVSValue blackness_args[] = { a, duration, color, color_yuv, colors };
+    static const char* const arg_names[5] = { 0, 0, "color", "color_yuv", "colors" };
+    return env->Invoke("BlankClip", AVSValue(blackness_args, 5), arg_names).AsClip();
   }
   else
   {
-    AVSValue blackness_args[] = { a, duration, color, fps };
-    static const char* const arg_names[4] = { 0, 0, "color", "fps" };
-    return env->Invoke("Blackness", AVSValue(blackness_args, 4), arg_names ).AsClip();
+    AVSValue blackness_args[] = { a, duration, color, fps, color_yuv, colors };
+    static const char* const arg_names[6] = { 0, 0, "color", "fps", "color_yuv", "colors" };
+    return env->Invoke("BlankClip", AVSValue(blackness_args, 6), arg_names).AsClip();
   }
 }
 
-AVSValue __cdecl Create_FadeOut0(AVSValue args, void*,IScriptEnvironment* env) {
+AVSValue __cdecl Create_Fade(AVSValue args, void* user_data, IScriptEnvironment* env) {
+  const int fade_type = (int)(intptr_t)user_data;
+
   const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
   const float fps = args[3].AsFloatf(24.0f);
+
+  int offset = 0;
+
   PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration,fadeclr,fps,env);
-  return new Dissolve(a, b, duration, fps, env);
+
+  switch (fade_type) {
+  case FADE_MODE_OUT0:
+  case FADE_MODE_IN0:
+  case FADE_MODE_IO0: offset = 0; break;
+  case FADE_MODE_OUT:
+  case FADE_MODE_IN:
+  case FADE_MODE_IO: offset = 1; break;
+  case FADE_MODE_OUT2:
+  case FADE_MODE_IN2:
+  case FADE_MODE_IO2: offset = 2; break;
+  }
+
+  PClip b = ColorClip(a, duration + offset, args[2] /* color */, args[4] /* color_yuv */, args[5] /* colors */, fps, env);
+
+  if(fade_type == FADE_MODE_OUT0 || fade_type == FADE_MODE_OUT || fade_type == FADE_MODE_OUT2)
+    return new Dissolve(a, b, duration, fps, env);
+  else if (fade_type == FADE_MODE_IN0 || fade_type == FADE_MODE_IN || fade_type == FADE_MODE_IN2)
+    return new Dissolve(b, a, duration, fps, env);
+  else {
+    /* IO In-Out */
+    AVSValue dissolve_args[] = { b, a, b, duration, fps };
+    return env->Invoke("Dissolve", AVSValue(dissolve_args, 5)).AsClip();
+  }
 }
-
-AVSValue __cdecl Create_FadeOut(AVSValue args, void*,IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration+1,fadeclr,fps,env);
-  return new Dissolve(a, b, duration, fps, env);
-}
-
-AVSValue __cdecl Create_FadeOut2(AVSValue args, void*,IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration+2,fadeclr,fps,env);
-  return new Dissolve(a, b, duration, fps, env);
-}
-
-AVSValue __cdecl Create_FadeIn0(AVSValue args, void*,IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration,fadeclr,fps,env);
-  return new Dissolve(b, a, duration, fps, env);
-}
-
-AVSValue __cdecl Create_FadeIn(AVSValue args, void*,IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration+1,fadeclr,fps,env);
-  return new Dissolve(b, a, duration, fps, env);
-}
-
-AVSValue __cdecl Create_FadeIn2(AVSValue args, void*,IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration+2,fadeclr,fps,env);
-  return new Dissolve(b, a, duration, fps, env);
-}
-
-AVSValue __cdecl Create_FadeIO0(AVSValue args, void*, IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration,fadeclr,fps,env);
-  AVSValue dissolve_args[] = { b, a, b, duration, fps };
-  return env->Invoke("Dissolve", AVSValue(dissolve_args,5)).AsClip();
-}
-
-AVSValue __cdecl Create_FadeIO(AVSValue args, void*, IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration+1,fadeclr,fps,env);
-  AVSValue dissolve_args[] = { b, a, b, duration, fps };
-  return env->Invoke("Dissolve", AVSValue(dissolve_args,5)).AsClip();
-}
-
-AVSValue __cdecl Create_FadeIO2(AVSValue args, void*, IScriptEnvironment* env) {
-  const int duration = args[1].AsInt();
-  const int fadeclr = args[2].AsInt(0);
-  const float fps = args[3].AsFloatf(24.0f);
-  PClip a = args[0].AsClip();
-  PClip b = ColorClip(a,duration+2,fadeclr,fps,env);
-  AVSValue dissolve_args[] = { b, a, b, duration, fps };
-  return env->Invoke("Dissolve", AVSValue(dissolve_args,5)).AsClip();
-}
-
-
-
