@@ -34,7 +34,8 @@
 
 
 #include "../core/internal.h"
-#include "../convert/convert.h"
+#include "../convert/convert_matrix.h"
+#include "../convert/convert_helper.h"
 #include "transform.h"
 #ifdef AVS_WINDOWS
 #include "AviSource/avi_source.h"
@@ -86,10 +87,36 @@ public:
       return 1;
     case CACHE_GET_MTMODE:
       return MT_NICE_FILTER;
+    case CACHE_GET_DEV_TYPE:
+       return DEV_TYPE_CPU;
+    case CACHE_GET_CHILD_DEV_TYPE:
+       return DEV_TYPE_ANY; // any type is ok because this clip does not require child's frames.
     default:
       return 0;
     }
-  };
+  }
+};
+
+// For any frame number, this clip returns the first frame of a child clip .
+// This clip makes cache effective and reduce unnecessary frame transfer.
+class SingleFrame : public GenericVideoFilter {
+public:
+  SingleFrame(PClip _child) : GenericVideoFilter(_child) {}
+  PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) { return child->GetFrame(0, env); }
+  int __stdcall SetCacheHints(int cachehints, int frame_range)
+  {
+    switch (cachehints)
+    {
+    case CACHE_DONT_CACHE_ME:
+      return 1;
+    case CACHE_GET_MTMODE:
+      return MT_NICE_FILTER;
+    case CACHE_GET_DEV_TYPE:
+      return (child->GetVersion() >= 5) ? child->SetCacheHints(CACHE_GET_DEV_TYPE, 0) : 0;
+    default:
+      return 0;
+    }
+  }
 };
 
 
@@ -100,6 +127,12 @@ static PVideoFrame CreateBlankFrame(const VideoInfo& vi, int color, int mode, co
   PVideoFrame frame = env->NewVideoFrame(vi);
   // no frame property origin
 
+  // but we set Rec601 (ST170) if YUV
+  auto props = env->getFramePropsRW(frame);
+  int theMatrix = vi.IsRGB() ? Matrix_e::AVS_MATRIX_RGB : Matrix_e::AVS_MATRIX_ST170_M;
+  int theColorRange = vi.IsRGB() ? ColorRange_e::AVS_RANGE_FULL : ColorRange_e::AVS_RANGE_LIMITED;
+  update_Matrix_and_ColorRange(props, theMatrix, theColorRange, env);
+
   // RGB 8->16 bit: not << 8 like YUV but 0..255 -> 0..65535 or 0..1023 for 10 bit
   int pixelsize = vi.ComponentSize();
   int bits_per_pixel = vi.BitsPerComponent();
@@ -134,7 +167,7 @@ static PVideoFrame CreateBlankFrame(const VideoInfo& vi, int color, int mode, co
       }
     }
     else {
-      int color_yuv = (mode == COLOR_MODE_YUV) ? color : RGB2YUV(color);
+      int color_yuv = (mode == COLOR_MODE_YUV) ? color : RGB2YUV_Rec601(color);
 
       int val_i = 0;
 
@@ -205,7 +238,7 @@ static PVideoFrame CreateBlankFrame(const VideoInfo& vi, int color, int mode, co
   int size = frame->GetPitch() * frame->GetHeight();
 
   if (vi.IsYUY2()) {
-    int color_yuv =(mode == COLOR_MODE_YUV) ? color : RGB2YUV(color);
+    int color_yuv =(mode == COLOR_MODE_YUV) ? color : RGB2YUV_Rec601(color);
     if (color_is_array) {
       color_yuv = (clamp(colors[0], 0, max_pixel_value) << 16) | (clamp(colors[1], 0, max_pixel_value) << 8) | (clamp(colors[2], 0, max_pixel_value));
     }
@@ -213,15 +246,16 @@ static PVideoFrame CreateBlankFrame(const VideoInfo& vi, int color, int mode, co
     for (int i=0; i<size; i+=4)
       *(uint32_t *)(p+i) = d;
   } else if (vi.IsRGB24()) {
-    const uint8_t color_b  = color_is_array ? clamp(colors[2], 0, max_pixel_value) : (uint8_t)(color & 0xFF);
-    const uint16_t color_rg = color_is_array ?
-                              ((clamp(colors[0], 0, max_pixel_value) << 8) | clamp(colors[1], 0, max_pixel_value)) :
-                              (uint16_t)(color >> 8);
+    const uint8_t color_b = color_is_array ? clamp(colors[2], 0, max_pixel_value) : (uint8_t)(color & 0xFF);
+    const uint8_t color_g = color_is_array ? clamp(colors[1], 0, max_pixel_value) : (uint8_t)(color >> 8);
+    const uint8_t color_r = color_is_array ? clamp(colors[0], 0, max_pixel_value) : (uint8_t)(color >> 16);
     const int rowsize = frame->GetRowSize();
     const int pitch = frame->GetPitch();
     for (int y=frame->GetHeight();y>0;y--) {
       for (int i=0; i<rowsize; i+=3) {
-        p[i] = color_b; *(uint16_t *)(p+i+1) = color_rg;
+        p[i] = color_b;
+        p[i+1] = color_g;
+        p[i+2] = color_r;
       }
       p+=pitch;
     }
@@ -288,11 +322,11 @@ static AVSValue __cdecl Create_BlankClip(AVSValue args, void*, IScriptEnvironmen
   AVSValue args0 = args[0];
 
   // param#12: "clip" overrides
-  if (args0.ArraySize() == 1 && !args[12].Defined()) {
+  if (args0.Defined() && args0.ArraySize() == 1 && !args[12].Defined()) {
     vi_default = args0[0].AsClip()->GetVideoInfo();
     parity = args0[0].AsClip()->GetParity(0);
   }
-  else if (args0.ArraySize() != 0) {
+  else if (args0.Defined() && args0.ArraySize() != 0) {
     // when "clip" is defined then beginning clip parameter is forbidden
     env->ThrowError("BlankClip: Only 1 Template clip allowed.");
   }
@@ -414,14 +448,12 @@ static AVSValue __cdecl Create_BlankClip(AVSValue args, void*, IScriptEnvironmen
       env->ThrowError("BlankClip: color_yuv only valid for YUV color spaces");
     color = args[11].AsInt();
     mode=COLOR_MODE_YUV;
-    if (!vi.IsYUVA() && (unsigned)color > 0xffffff)
-      env->ThrowError("BlankClip: color_yuv must be between 0 and %d($ffffff)", 0xffffff);
   }
 
   int colors[4] = { 0 };
   float colors_f[4] = { 0.0 };
   bool color_is_array = false;
-#ifdef NEW_AVSVALUE
+
   if (args.ArraySize() >= 14) {
     // new colors parameter
     if (args[13].Defined()) // colors
@@ -429,8 +461,8 @@ static AVSValue __cdecl Create_BlankClip(AVSValue args, void*, IScriptEnvironmen
       if (!args[13].IsArray())
         env->ThrowError("BlankClip: colors must be an array");
       int color_count = args[13].ArraySize();
-      if (vi.NumComponents() != color_count)
-        env->ThrowError("BlankClip: color count %d does not match to component count %d", color_count, vi.NumComponents());
+      if (color_count < vi.NumComponents())
+        env->ThrowError("BlankClip: 'colors' size %d is less than component count %d", color_count, vi.NumComponents());
       int pixelsize = vi.ComponentSize();
       int bits_per_pixel = vi.BitsPerComponent();
       for (int i = 0; i < color_count; i++) {
@@ -447,9 +479,12 @@ static AVSValue __cdecl Create_BlankClip(AVSValue args, void*, IScriptEnvironmen
       color_is_array = true;
     }
   }
-#endif
 
-  return new StaticImage(vi, CreateBlankFrame(vi, color, mode, colors, colors_f, color_is_array, env), parity);
+  PClip clip = new StaticImage(vi, CreateBlankFrame(vi, color, mode, colors, colors_f, color_is_array, env), parity);
+
+  // wrap in OnCPU to support multi devices
+  AVSValue arg[2]{ clip, 1 }; // prefetch=1: enable cache but not thread
+  return new SingleFrame(env->Invoke("OnCPU", AVSValue(arg, 2)).AsClip());
 }
 
 
@@ -467,7 +502,9 @@ extern bool GetTextBoundingBoxFixed(const char* text, const char* fontname, int
 
 
 PClip Create_MessageClip(const char* message, int width, int height, int pixel_type, bool shrink,
-                         int textcolor, int halocolor, int bgcolor, IScriptEnvironment* env) {
+                         int textcolor, int halocolor, int bgcolor,
+                         int fps_numerator, int fps_denominator, int num_frames,
+                         IScriptEnvironment* env) {
   int size;
 #if defined(AVS_WINDOWS) && !defined(NO_WIN_GDI)
     // MessageClip produces a clip containing a text message.Used internally for error reporting.
@@ -519,19 +556,25 @@ PClip Create_MessageClip(const char* message, int width, int height, int pixel_t
   vi.width = width;
   vi.height = height;
   vi.pixel_type = pixel_type;
-  vi.fps_numerator = 24;
-  vi.fps_denominator = 1;
-  vi.num_frames = 240;
+  vi.fps_numerator = fps_numerator > 0 ? fps_numerator : 24;
+  vi.fps_denominator = fps_denominator > 0 ? fps_denominator : 1;
+  vi.num_frames = num_frames > 0 ? num_frames : 240;
 
   PVideoFrame frame = CreateBlankFrame(vi, bgcolor, COLOR_MODE_RGB, nullptr, nullptr, false, env);
   env->ApplyMessage(&frame, vi, message, size, textcolor, halocolor, bgcolor);
-  return new StaticImage(vi, frame, false);
+  PClip clip = new StaticImage(vi, frame, false);
+
+  // wrap in OnCPU to support multi devices
+  AVSValue args[2]{ clip, 1 }; // prefetch=1: enable cache but not thread
+  return new SingleFrame(env->Invoke("OnCPU", AVSValue(args, 2)).AsClip());
 };
 
 AVSValue __cdecl Create_MessageClip(AVSValue args, void*, IScriptEnvironment* env) {
   return Create_MessageClip(args[0].AsString(), args[1].AsInt(-1),
       args[2].AsInt(-1), VideoInfo::CS_BGR32, args[3].AsBool(false),
-      args[4].AsInt(0xFFFFFF), args[5].AsInt(0), args[6].AsInt(0), env);
+      args[4].AsInt(0xFFFFFF), args[5].AsInt(0), args[6].AsInt(0),
+      -1, -1, -1, // fps_numerator, fps_denominator, num_frames: auto
+    env);
 }
 
 
@@ -560,7 +603,7 @@ static void draw_colorbars_444(uint8_t *pY8, uint8_t *pU8, uint8_t *pV8, int pit
   typedef typename std::conditional < sizeof(pixel_t) == 4, float, int>::type factor_t; // float support
   factor_t factor = (pixel_t)(sizeof(pixel_t) == 4 ? 1 / 255.0f : 1);
 
-  //                                              LtGrey  Yellow    Cyan   Green Magenta     Red    Blue
+  //                                              LtGrey  Yellow    Cyan   Green Magenta     Red    Blue (all 75%, BT.601)
   static const BYTE top_two_thirdsY[] = { 0xb4,   0xa2,   0x83,   0x70,   0x54,   0x41,   0x23 };
   static const BYTE top_two_thirdsU[] = { 0x80,   0x2c,   0x9c,   0x48,   0xb8,   0x64,   0xd4 };
   static const BYTE top_two_thirdsV[] = { 0x80,   0x8e,   0x2c,   0x3a,   0xc6,   0xd4,   0x72 };
@@ -833,6 +876,77 @@ static void draw_colorbarsHD_444(uint8_t *pY8, uint8_t *pU8, uint8_t *pV8, int p
 //		1288 x  720 <- default
 //		1456 x 1080  hd anamorphic
 //		1904 x 1080
+/*
+  ARIB STD-B28  Version 1.0-E1
+
+  *1: 75W/100W/I+: Choice from 75% white, 100% white and +I signal
+  *2: can be changed to any value other than the standard values in accordance with the operation purpose by the user
+
+              |<-------------------------------------------------------- a -------------------------------------------------->|
+              |             |<------------------------------------------3/4 a --------------------------------->|             |
+              |<---- d ---->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<---- d ---->|
+
+              +-------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+-------------+  -----------
+              |             |           |           |           |           |           |           |           |             |      ^   ^
+              |             |           |           |           |           |           |           |           |             |      |   |
+Pattern 1     |     40%     |    75%    |    75%    |    75%    |    75%    |    75%    |    75%    |    75%    |    40%      |      |   |
+              |    Grey     |   White   |   Yellow  |   Cyan    |   Green   |  Magenta  |   Red     |   Blue    |    Grey     | 7/12b|   |
+              |     *2      |           |           |           |           |           |           |           |    *2       |      |   |
+              |             |           |           |           |           |           |           |           |             |      V   |
+              +-------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+-------------+ -------  |
+Pattern 2     | 100% Cyan   |75W/100W/I+|                          75% white (chroma set signal)                | 100% blue   | 1/12b|   | b
+              +-------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+-------------+ ------   |
+Pattern 3     | 100% Yellow |                                       Y ramp                                      | 100% red    | 1/12b|   |
+              +-------------+-----------------+-----------+-----------+-------+----+----+---+---+---+-----------+-------------+ -------  |
+              |     *2      |                 |                       |       |    |    |   |   |   |           |      *2     |      ^   |
+Pattern 4     |     15%     |        0%       |          100%         |   0%  |-2% | 0  |+2%| 0 |+4%|     0%    |     15%     | 3/12 |   |
+              |    Grey     |       Black     |         White         | Black |    |    |   |   |   |   Black   |    Grey     |  b   V   V
+              +-------------+-----------------+-----------+-----------+-------+----+----+---+---+---+-----------+-------------+  -----------
+
+              |<---- d ---->|<---- 3/2 c ---->|<--------- 2c -------->|<5/6c->|1/3c|1/3c|1/3|1/3|1/3|<--- c --->|<---- d ---->|
+
+              a:b = 16:9
+
+
+  2021: SMPTE RP 219-1:2014
+  *1: can be changed to any value other than the standard values in accordance with the operation purpose by the user
+  *2: 75W/100W/+I/-I: Choice from 75% white, 100% white and +I or -I signal
+  *3: 0% Black or +Q (Left from Y ramp)
+  *4: can be changed to any value other than the standard values in accordance with the operation purpose by the user
+  *5: Choice from 0% Black, Sub-black valley
+      The sub-black valley signal shall begin at the 0% black level, shall decrease in a linear ramp to the minimum permitted level at the mid-point,
+      and shall increase in a linear ramp to the 0% black level at the end of the black bar.
+  *6: Choice from 100% White, Super-white Peak
+      The super-white peak signal shall begin at the 100% white level, shall increase in a linear ramp to the maximum permitted level at the midpoint, 
+      and shall decrease in a linear ramp to the 100% white level at the end of the white bar. 
+
+                            |<-------------------------------------------------------- a -------------------------------------------------->|
+              |             |<------------------------------------------3/4 a --------------------------------->|             |
+              |<---- d ---->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<--- c --->|<---- d ---->|
+
+              +-------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+-------------+  -----------
+              |             |           |           |           |           |           |           |           |             |      ^   ^
+              |             |           |           |           |           |           |           |           |             |      |   |
+Pattern 1     |     40%     |    75%    |    75%    |    75%    |    75%    |    75%    |    75%    |    75%    |    40%      |      |   |
+              |    Grey     |   White   |   Yellow  |   Cyan    |   Green   |  Magenta  |   Red     |   Blue    |    Grey     | 7/12b|   |
+              |     *1      |           |           |           |           |           |           |           |    *1       |      |   |
+              |             |           |           |           |           |           |           |           |             |      V   |
+              +-------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+-------------+ -------  |
+Pattern 2     | 100% Cyan   |75/100W/I-+|                          75% white (chroma set signal)                | 100% blue   | 1/12b|   | b
+              +-------------+-----------+-----------+-----------+-----------+-----------+-----------+-----------+-------------+ ------   |
+Pattern 3     | 100% Yellow |0%Blk or +Q|                           Y ramp                          | 100% White| 100% red    | 1/12b|   |
+              +-------------+-----------+-----+-----------+-----------+-------+----+----+---+---+---+-----------+-------------+ -------  |
+              |     *4      |   0% Black    *5|      100% White     *6|       |    |    |   |   |   |           |      *4     |      ^   |
+Pattern 4     |     15%     |0% Blk or SubBlck|100%White/SuperWhtePeak|   0%  |-2% | 0  |+2%| 0 |+4%|     0%    |     15%     | 3/12 |   |
+              |    Grey     |   0%  Black     |      100% White       | Black |    |    |   |   |   |   Black   |    Grey     |  b   V   V
+              +-------------+-----------------+-----------+-----------+-------+----+----+---+---+---+-----------+-------------+  -----------
+
+              |<---- d ---->|<---- 3/2 c ---->|<--------- 2c -------->|<5/6c->|1/3c|1/3c|1/3|1/3|1/3|<--- c --->|<---- d ---->|
+
+              a:b = 16:9
+
+*/
+
   int y = 0;
 
   const int c = (w * 3 + 14) / 28; // 1/7th of 3/4 of width
@@ -843,7 +957,7 @@ static void draw_colorbarsHD_444(uint8_t *pY8, uint8_t *pU8, uint8_t *pV8, int p
   const int p1 = h - p23 * 2 - p4; // remaining 7/12th of height
 
   //               75%  Rec709 -- Grey40 Grey75 Yellow  Cyan   Green Magenta  Red   Blue
-  static const BYTE pattern1Y[] = { 104,   180,   168,   145,   134,    63,    51,    28 };
+  static const BYTE pattern1Y[] = { 104,   180,   168,   145,   133,    63,    51,    28 };
   static const BYTE pattern1U[] = { 128,   128,    44,   147,    63,   193,   109,   212 };
   static const BYTE pattern1V[] = { 128,   128,   136,    44,    52,   204,   212,   120 };
   for (; y < p1; ++y) { // Pattern 1
@@ -886,9 +1000,9 @@ static void draw_colorbarsHD_444(uint8_t *pY8, uint8_t *pU8, uint8_t *pV8, int p
     pY += pitchY; pU += pitchUV; pV += pitchUV;
   }
   //              100% Rec709       Cyan  Blue Yellow  Red    +I Grey75  White
-  static const BYTE pattern23Y[] = { 188,   32,  219,   63,   16,  180,  235 };
-  static const BYTE pattern23U[] = { 154,  240,   16,  102,   98,  128,  128 };
-  static const BYTE pattern23V[] = {  16,  118,  138,  240,  161,  128,  128 };
+  static const BYTE pattern23Y[] = { 188,   32,  219,   63,   61,  180,  235 };
+  static const BYTE pattern23U[] = { 154,  240,   16,  102,  103,  128,  128 };
+  static const BYTE pattern23V[] = {  16,  118,  138,  240,  157,  128,  128 };
   for (; y < p1 + p23; ++y) { // Pattern 2
     int x = 0;
     for (; x < d; ++x) {
@@ -950,6 +1064,7 @@ static void draw_colorbarsHD_444(uint8_t *pY8, uint8_t *pU8, uint8_t *pV8, int p
         pV[x] = factor * (pattern23V[2] << shift);
       }
     }
+    // FIXME: Y-ramp to conform SMPTE RP 219-1:2014, put 0% Black before and 100% White after
     for (int j = 0; j < c * 7; ++j, ++x) { // Y-Ramp
       pY[x] = pixel_t(factor*(16 << shift) + (factor * (220 << shift) * j) / (c * 7));
       if constexpr(sizeof(pixel_t) == 4) {
@@ -1035,6 +1150,17 @@ static uint64_t rgbcolor32to64(uint32_t color32_8)
     );
 }
 
+// Studio RGB constants for ColorBars
+static const uint32_t bottom_quarter[] =
+// RGB[16..235]     -I     white        +Q     Black     -4% Black     Black     +4% Black     Black
+{ 0x10466a, 0xebebeb, 0x481076, 0x101010,  0x070707, 0x101010, 0x191919,  0x101010 }; // Qlum=Ilum=13.4%
+static const int two_thirds_to_three_quarters[] =
+// RGB[16..235]   Blue     Black  Magenta      Black      Cyan     Black    LtGrey
+{ 0x1010b4, 0x101010, 0xb410b4, 0x101010, 0x10b4b4, 0x101010, 0xb4b4b4 };
+static const int top_two_thirds[] =
+// RGB[16..235] LtGrey    Yellow      Cyan     Green   Magenta       Red      Blue
+{ 0xb4b4b4, 0xb4b410, 0x10b4b4, 0x10b410, 0xb410b4, 0xb41010, 0x1010b4 };
+
 template<typename pixel_t>
 static void draw_colorbars_rgb3264(uint8_t *p8, int pitch, int w, int h)
 {
@@ -1044,9 +1170,6 @@ static void draw_colorbars_rgb3264(uint8_t *p8, int pitch, int w, int h)
   pitch /= sizeof(pixel_t);
 
   // note we go bottom->top
-  static const uint32_t bottom_quarter[] =
-  // RGB[16..235]     -I     white        +Q     Black     -4ire     Black     +4ire     Black
-  { 0x003a62, 0xebebeb, 0x4b0f7e, 0x101010,  0x070707, 0x101010, 0x191919,  0x101010 }; // Qlum=Ilum=13.4%
 
   int y = 0;
 
@@ -1070,9 +1193,6 @@ static void draw_colorbars_rgb3264(uint8_t *p8, int pitch, int w, int h)
     p += pitch;
   }
 
-  static const int two_thirds_to_three_quarters[] =
-  // RGB[16..235]   Blue     Black  Magenta      Black      Cyan     Black    LtGrey
-  { 0x1010b4, 0x101010, 0xb410b4, 0x101010, 0x10b4b4, 0x101010, 0xb4b4b4 };
   for (; y < h / 3; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1082,9 +1202,6 @@ static void draw_colorbars_rgb3264(uint8_t *p8, int pitch, int w, int h)
     p += pitch;
   }
 
-  static const int top_two_thirds[] =
-  // RGB[16..235] LtGrey    Yellow      Cyan     Green   Magenta       Red      Blue
-  { 0xb4b4b4, 0xb4b410, 0x10b4b4, 0x10b410, 0xb410b4, 0xb41010, 0x1010b4 };
   for (; y < h; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1104,10 +1221,6 @@ static void draw_colorbars_rgb2448(uint8_t* p8, int pitch, int w, int h)
   pitch /= sizeof(pixel_t);
 
   // note we go bottom->top
-  static const uint32_t bottom_quarter[] =
-    // RGB[16..235]     -I     white        +Q     Black     -4ire     Black     +4ire     Black
-  { 0x003a62, 0xebebeb, 0x4b0f7e, 0x101010,  0x070707, 0x101010, 0x191919,  0x101010 }; // Qlum=Ilum=13.4%
-
   int y = 0;
 
   constexpr bool isRGB24 = sizeof(pixel_t) == 1;
@@ -1141,9 +1254,6 @@ static void draw_colorbars_rgb2448(uint8_t* p8, int pitch, int w, int h)
     p += pitch;
   }
 
-  static const int two_thirds_to_three_quarters[] =
-    // RGB[16..235]   Blue     Black  Magenta      Black      Cyan     Black    LtGrey
-  { 0x1010b4, 0x101010, 0xb410b4, 0x101010, 0x10b4b4, 0x101010, 0xb4b4b4 };
   for (; y < h / 3; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1158,9 +1268,6 @@ static void draw_colorbars_rgb2448(uint8_t* p8, int pitch, int w, int h)
     p += pitch;
   }
 
-  static const int top_two_thirds[] =
-    // RGB[16..235] LtGrey    Yellow      Cyan     Green   Magenta       Red      Blue
-  { 0xb4b4b4, 0xb4b410, 0x10b4b4, 0x10b410, 0xb410b4, 0xb41010, 0x1010b4 };
   for (; y < h; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1190,10 +1297,6 @@ static void draw_colorbars_rgbp(uint8_t *pR8, uint8_t *pG8, uint8_t *pB8, int pi
   pG += (h - 1)*pitch;
   pB += (h - 1)*pitch;
 
-  static const uint32_t bottom_quarter[] =
-    // RGB[16..235]     -I     white        +Q     Black     -4ire     Black     +4ire     Black
-  { 0x003a62, 0xebebeb, 0x4b0f7e, 0x101010,  0x070707, 0x101010, 0x191919,  0x101010 }; // Qlum=Ilum=13.4%
-
   int y = 0;
 
   const int max_pixel_value = (1 << bits_per_pixel) - 1;
@@ -1256,9 +1359,6 @@ static void draw_colorbars_rgbp(uint8_t *pR8, uint8_t *pG8, uint8_t *pB8, int pi
     pB -= pitch;
   }
 
-  static const int two_thirds_to_three_quarters[] =
-    // RGB[16..235]   Blue     Black  Magenta      Black      Cyan     Black    LtGrey
-  { 0x1010b4, 0x101010, 0xb410b4, 0x101010, 0x10b4b4, 0x101010, 0xb4b4b4 };
   for (; y < h / 3; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1282,10 +1382,6 @@ static void draw_colorbars_rgbp(uint8_t *pR8, uint8_t *pG8, uint8_t *pB8, int pi
     pB -= pitch;
   }
 
-  static const int top_two_thirds[] =
-    // RGB[16..235] LtGrey    Yellow      Cyan     Green   Magenta       Red      Blue
-  { 0xb4b4b4, 0xb4b410, 0x10b4b4, 0x10b410, 0xb410b4, 0xb41010, 0x1010b4 };
-
   for (; y < h; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1324,10 +1420,6 @@ static void draw_colorbars_rgbp_f(uint8_t *pR8, uint8_t *pG8, uint8_t *pB8, int
   pG += (h - 1)*pitch;
   pB += (h - 1)*pitch;
 
-  static const uint32_t bottom_quarter[] =
-  // RGB[16..235]     -I     white        +Q     Black     -4ire     Black     +4ire     Black
-  { 0x003a62, 0xebebeb, 0x4b0f7e, 0x101010,  0x070707, 0x101010, 0x191919,  0x101010 }; // Qlum=Ilum=13.4%
-
   int y = 0;
 
   const float factor = 1.0f / 255;
@@ -1373,9 +1465,6 @@ static void draw_colorbars_rgbp_f(uint8_t *pR8, uint8_t *pG8, uint8_t *pB8, int
     pB -= pitch;
   }
 
-  static const int two_thirds_to_three_quarters[] =
-    // RGB[16..235]   Blue     Black  Magenta      Black      Cyan     Black    LtGrey
-  { 0x1010b4, 0x101010, 0xb410b4, 0x101010, 0x10b4b4, 0x101010, 0xb4b4b4 };
   for (; y < h / 3; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1394,10 +1483,6 @@ static void draw_colorbars_rgbp_f(uint8_t *pR8, uint8_t *pG8, uint8_t *pB8, int
     pB -= pitch;
   }
 
-  static const int top_two_thirds[] =
-    // RGB[16..235] LtGrey    Yellow      Cyan     Green   Magenta       Red      Blue
-  { 0xb4b4b4, 0xb4b410, 0x10b4b4, 0x10b410, 0xb410b4, 0xb41010, 0x1010b4 };
-
   for (; y < h; ++y) {
     int x = 0;
     for (int i = 0; i < 7; ++i) {
@@ -1482,11 +1567,28 @@ public:
     vi.num_audio_samples=vi.AudioSamplesFromFrames(vi.num_frames);
 
     frame = env->NewVideoFrame(vi);
-    // FIXME: set colorimetry frame properties
+
     uint32_t* p = (uint32_t *)frame->GetWritePtr();
 
     int y = 0;
 
+    // set basic frame properties
+    auto props = env->getFramePropsRW(frame);
+    int theMatrix;
+    int theColorRange;
+    if (type) {
+      // ColorBarsHD 444 only
+      theMatrix = Matrix_e::AVS_MATRIX_BT709;
+      theColorRange = ColorRange_e::AVS_RANGE_LIMITED;
+    }
+    else {
+      // ColorBars can be rgb or yuv
+      theMatrix = vi.IsRGB() ? Matrix_e::AVS_MATRIX_RGB : Matrix_e::AVS_MATRIX_BT709;
+      // Studio RGB: limited!
+      theColorRange = vi.IsRGB() ? ColorRange_e::AVS_RANGE_LIMITED : ColorRange_e::AVS_RANGE_LIMITED;
+    }
+    update_Matrix_and_ColorRange(props, theMatrix, theColorRange, env);
+
 	// HD colorbars arib_std_b28
 	// Rec709 yuv values calculated by jmac698, Jan 2010, for Midzuki
 	if (type) { // ColorbarsHD
@@ -1554,7 +1656,7 @@ public:
   }
   else if (vi.IsYUY2()) {
     const int pitch = frame->GetPitch() / 4;
-    static const unsigned int top_two_thirds[] =
+    static const unsigned int top_two_thirds_yuy2[] =
 //                LtGrey      Yellow        Cyan       Green     Magenta         Red        Blue
     { 0x80b480b4, 0x8ea22ca2, 0x2c839c83, 0x3a704870, 0xc654b854, 0xd4416441, 0x7223d423 }; //VYUY
     w >>= 1;
@@ -1562,38 +1664,38 @@ public:
       int x = 0;
       for (int i = 0; i < 7; i++) {
         for (; x < (w*(i + 1) + 3) / 7; ++x)
-          p[x] = top_two_thirds[i];
+          p[x] = top_two_thirds_yuy2[i];
       }
       p += pitch;
     }
 
-    static const unsigned  int two_thirds_to_three_quarters[] =
+    static const unsigned  int two_thirds_to_three_quarters_yuy2[] =
 //                 Blue        Black     Magenta       Black        Cyan       Black      LtGrey
     { 0x7223d423, 0x80108010, 0xc654b854, 0x80108010, 0x2c839c83, 0x80108010, 0x80b480b4 }; //VYUY
     for (; y * 4 < h * 3; ++y) {
       int x = 0;
       for (int i = 0; i < 7; i++) {
         for (; x < (w*(i + 1) + 3) / 7; ++x)
-          p[x] = two_thirds_to_three_quarters[i];
+          p[x] = two_thirds_to_three_quarters_yuy2[i];
       }
       p += pitch;
     }
 
-    static const unsigned int bottom_quarter[] =
+    static const unsigned int bottom_quarter_yuy2[] =
 //                    -I       white          +Q       Black       -4ire       Black       +4ire       Black
     { 0x5f109e10, 0x80eb80eb, 0x9510ae10, 0x80108010, 0x80078007, 0x80108010, 0x80198019, 0x80108010 }; //VYUY
     for (; y < h; ++y) {
       int x = 0;
       for (int i = 0; i < 4; ++i) {
         for (; x < (w*(i + 1) * 5 + 14) / 28; ++x)
-          p[x] = bottom_quarter[i];
+          p[x] = bottom_quarter_yuy2[i];
       }
       for (int j = 4; j < 7; ++j) {
         for (; x < (w*(j + 12) + 10) / 21; ++x)
-          p[x] = bottom_quarter[j];
+          p[x] = bottom_quarter_yuy2[j];
       }
       for (; x < w; ++x)
-        p[x] = bottom_quarter[7];
+        p[x] = bottom_quarter_yuy2[7];
       p += pitch;
     }
   }
@@ -1712,9 +1814,35 @@ public:
       }
   };
 
+  void FillAudioZeros(void* buf, int start_offset, int count) {
+    const int bps = vi.BytesPerAudioSample();
+    unsigned char* byte_buf = (unsigned char*)buf;
+    memset(byte_buf + start_offset * bps, 0, count * bps);
+  }
+
   void __stdcall GetAudio(void* buf, int64_t start, int64_t count, IScriptEnvironment* env) {
     AVS_UNUSED(env);
 #if 1
+    // This filter is non-cached so we guard against negative start and overread, like in Cache::GetAudio
+    if ((start + count <= 0) || (start >= vi.num_audio_samples)) {
+      // Completely skip.
+      FillAudioZeros(buf, 0, (int)count);
+      count = 0;
+      return;
+    }
+
+    if (start < 0) {  // Partial initial skip
+      FillAudioZeros(buf, 0, (int)-start);  // Fill all samples before 0 with silence.
+      count += start;  // Subtract start bytes from count.
+      buf = ((BYTE*)buf) - (int)(start * vi.BytesPerAudioSample());
+      start = 0;
+    }
+
+    if (start + count > vi.num_audio_samples) {  // Partial ending skip
+      FillAudioZeros(buf, (int)(vi.num_audio_samples - start), (int)(count - (vi.num_audio_samples - start)));  // Fill end samples
+      count = (vi.num_audio_samples - start);
+    }
+
 	const int d_mod = vi.audio_samples_per_second*2;
 	float* samples = (float*)buf;
 
@@ -1756,12 +1884,20 @@ public:
 
   static AVSValue __cdecl Create(AVSValue args, void* _type, IScriptEnvironment* env) {
     const int type = (int)(size_t)_type;
+    bool staticframes = args[3].AsBool(true);
 
-    return new ColorBars(args[0].AsInt(   type ? 1288 : 640),
+    PClip clip = new ColorBars(args[0].AsInt(   type ? 1288 : 640),
                          args[1].AsInt(   type ?  720 : 480),
                          args[2].AsString(type ? "YV24" : "RGB32"),
-                         args[3].AsBool(true), // new staticframes parameter
+                         staticframes, // new staticframes parameter
                          type, env);
+    // wrap in OnCPU to support multi devices
+    AVSValue arg[2]{ clip, 1 }; // prefetch=1: enable cache but not thread
+    AVSValue ret = env->Invoke("OnCPU", AVSValue(arg, 2));
+    if (staticframes) {
+      return new SingleFrame(ret.AsClip());
+    }
+    return ret;
   }
 };
 
@@ -2003,13 +2139,43 @@ public:
 
 
 AVSValue __cdecl Create_Version(AVSValue args, void*, IScriptEnvironment* env) {
+  //     0       1       2           3         4
+  // [length]i[width]i[height]i[pixel_type]s[clip]c
+  VideoInfo vi_default;
+
+  int i_pixel_type = VideoInfo::CS_BGR24;
+
+  const bool has_clip = args[4].Defined();
+  if (has_clip) {
+    // clip overrides
+    vi_default = args[4].AsClip()->GetVideoInfo();
+    i_pixel_type = vi_default.pixel_type;
+  }
+
+  if (args[3].Defined()) {
+    i_pixel_type = GetPixelTypeFromName(args[3].AsString());
+    if (i_pixel_type == VideoInfo::CS_UNKNOWN)
+      env->ThrowError("Version: invalid 'pixel_type'");
+  }
+
+  int num_frames = args[0].AsInt(has_clip ? vi_default.num_frames : -1); // auto (240)
+  int w = args[1].AsInt(has_clip ? vi_default.width : -1); // auto
+  int h = args[2].AsInt(has_clip ? vi_default.height : -1); // auto
+  const bool shrink = false;
+  const int textcolor = 0xECF2BF;
+  const int halocolor = 0;
+  const int bgcolor = 0x404040;
+
+  const int fps_numerator = has_clip ? vi_default.fps_numerator :-1; // auto
+  const int fps_denominator = has_clip ? vi_default.fps_denominator : -1; // auto
+
   return Create_MessageClip(
 #ifdef AVS_POSIX
     AVS_FULLVERSION AVS_COPYRIGHT_UTF8
 #else
     AVS_FULLVERSION AVS_COPYRIGHT
 #endif
-    ,-1, -1, VideoInfo::CS_BGR24, false, 0xECF2BF, 0, 0x404040, env);
+    , w, h, i_pixel_type, shrink, textcolor, halocolor, bgcolor, fps_numerator, fps_denominator, num_frames, env);
 }
 
 
@@ -2028,10 +2194,8 @@ extern const AVSFunction Source_filters[] = {
 #endif
   { "BlankClip", BUILTIN_FUNC_PREFIX, "[]c*[length]i[width]i[height]i[pixel_type]s[fps]f[fps_denominator]i[audio_rate]i[stereo]b[sixteen_bit]b[color]i[color_yuv]i[clip]c", Create_BlankClip },
   { "BlankClip", BUILTIN_FUNC_PREFIX, "[]c*[length]i[width]i[height]i[pixel_type]s[fps]f[fps_denominator]i[audio_rate]i[channels]i[sample_type]s[color]i[color_yuv]i[clip]c", Create_BlankClip },
-#ifdef NEW_AVSVALUE
   { "BlankClip", BUILTIN_FUNC_PREFIX, "[]c*[length]i[width]i[height]i[pixel_type]s[fps]f[fps_denominator]i[audio_rate]i[stereo]b[sixteen_bit]b[color]i[color_yuv]i[clip]c[colors]f+", Create_BlankClip },
   { "BlankClip", BUILTIN_FUNC_PREFIX, "[]c*[length]i[width]i[height]i[pixel_type]s[fps]f[fps_denominator]i[audio_rate]i[channels]i[sample_type]s[color]i[color_yuv]i[clip]c[colors]f+", Create_BlankClip },
-#endif
   { "Blackness", BUILTIN_FUNC_PREFIX, "[]c*[length]i[width]i[height]i[pixel_type]s[fps]f[fps_denominator]i[audio_rate]i[stereo]b[sixteen_bit]b[color]i[color_yuv]i[clip]c", Create_BlankClip },
   { "Blackness", BUILTIN_FUNC_PREFIX, "[]c*[length]i[width]i[height]i[pixel_type]s[fps]f[fps_denominator]i[audio_rate]i[channels]i[sample_type]s[color]i[color_yuv]i[clip]c", Create_BlankClip },
   { "MessageClip", BUILTIN_FUNC_PREFIX, "s[width]i[height]i[shrink]b[text_color]i[halo_color]i[bg_color]i", Create_MessageClip },
@@ -2039,7 +2203,7 @@ extern const AVSFunction Source_filters[] = {
   { "ColorBarsHD", BUILTIN_FUNC_PREFIX, "[width]i[height]i[pixel_type]s[staticframes]b", ColorBars::Create, (void*)1 },
   { "Tone", BUILTIN_FUNC_PREFIX, "[length]f[frequency]f[samplerate]i[channels]i[type]s[level]f", Tone::Create },
 
-  { "Version", BUILTIN_FUNC_PREFIX, "", Create_Version },
+  { "Version", BUILTIN_FUNC_PREFIX, "[length]i[width]i[height]i[pixel_type]s[clip]c", Create_Version },
 
   { NULL }
 };
